@@ -10,8 +10,8 @@
 %%
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
-%%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
+%%   The Initial Developer of the Original Code is GoPivotal, Inc.
+%%   Copyright (c) 2010-2013 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_util).
@@ -257,7 +257,7 @@ id(Key, ReqData) ->
     id0(Key, ReqData).
 
 id0(Key, ReqData) ->
-    case dict:find(Key, wrq:path_info(ReqData)) of
+    case orddict:find(Key, wrq:path_info(ReqData)) of
         {ok, Id} -> list_to_binary(mochiweb_util:unquote(Id));
         error    -> none
     end.
