@@ -10,8 +10,8 @@
 %%
 %% The Original Code is RabbitMQ.
 %%
-%% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
+%% The Initial Developer of the Original Code is GoPivotal, Inc.
+%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_stomp_processor).
@@ -138,7 +138,8 @@ handle_info({Delivery = #'basic.deliver'{},
              #amqp_msg{props = Props, payload = Payload}}, State) ->
     {noreply, send_delivery(Delivery, Props, Payload, State), hibernate};
 handle_info(#'basic.cancel'{consumer_tag = Ctag}, State) ->
-    {noreply, server_cancel_consumer(Ctag, State), hibernate};
+    process_request(
+      fun(StateN) -> server_cancel_consumer(Ctag, StateN) end, State);
 handle_info({'EXIT', Conn,
              {shutdown, {server_initiated_close, Code, Explanation}}},
             State = #state{connection = Conn}) ->
@@ -154,6 +155,9 @@ handle_info({bump_credit, Msg}, State) ->
 handle_info({inet_reply, _, Status}, State) ->
     {stop, Status, State}.
 
+process_request(ProcessFun, State) ->
+    process_request(ProcessFun, fun (StateM) -> StateM end, State).
+
 process_request(ProcessFun, SuccessFun, State) ->
     Res = case catch ProcessFun(State) of
               {'EXIT',
@@ -214,7 +218,6 @@ process_connect(Implicit, Frame,
                             StateN)
               end
       end,
-      fun(StateM) -> StateM end,
       State).
 
 creds(Frame, SSLLoginName,
